/* $Id: Tone.h 113 2010-06-16 20:16:29Z bhagman@roguerobotics.com $

  A Tone Generator Library

  Written by Brett Hagman
  http://www.roguerobotics.com/
  bhagman@roguerobotics.com

    This library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*************************************************/

#ifndef _Tone_h
#define _Tone_h

#include <stdint.h>

/*************************************************
* Public Constants
*************************************************/

// Define all notes according to their frequencies
// refer to wikipedia Piano key frequencies
// (rounded to nearest integer)
// Note: S denotes 'sharp', F denotes 'flat'
#define REST     0 
#define NOTE_A0  28
#define NOTE_A0S 29
#define NOTE_B0F 29
#define NOTE_B0  31
#define NOTE_C1F 31
#define NOTE_B0S 33
#define NOTE_C1  33
#define NOTE_C1S 35
#define NOTE_D1F 35
#define NOTE_D1  37
#define NOTE_D1S 39
#define NOTE_E1F 39
#define NOTE_E1  41
#define NOTE_F1F 41
#define NOTE_E1S 44
#define NOTE_F1  44
#define NOTE_F1S 46
#define NOTE_G1F 46
#define NOTE_G1  49
#define NOTE_G1S 52
#define NOTE_A1F 52
#define NOTE_A1  55
#define NOTE_A1S 58
#define NOTE_B1F 58
#define NOTE_B1  62
#define NOTE_C2F 62
#define NOTE_B2S 65
#define NOTE_C2  65
#define NOTE_C2S 69
#define NOTE_D2F 69
#define NOTE_D2  73
#define NOTE_D2S 78
#define NOTE_E2F 78
#define NOTE_E2  82
#define NOTE_F2F 82
#define NOTE_E2S 87
#define NOTE_F2  87
#define NOTE_F2S 93
#define NOTE_G2F 93
#define NOTE_G2  98
#define NOTE_G2S 104
#define NOTE_A2F 104
#define NOTE_A2  110
#define NOTE_A2S 117
#define NOTE_B2F 117
#define NOTE_B2  123
#define NOTE_C3F 123
#define NOTE_B2S 131
#define NOTE_C3  131
#define NOTE_C3S 139
#define NOTE_D3F 139
#define NOTE_D3  147
#define NOTE_D3S 156
#define NOTE_E3F 156
#define NOTE_E3  165
#define NOTE_F3F 165
#define NOTE_E3S 175
#define NOTE_F3  175
#define NOTE_F3S 185
#define NOTE_G3F 185
#define NOTE_G3  196
#define NOTE_G3S 208
#define NOTE_A3F 208
#define NOTE_A3  220
#define NOTE_A3S 233
#define NOTE_B3F 233
#define NOTE_B3  247
#define NOTE_C4F 247
#define NOTE_B3S 262
#define NOTE_C4  262
#define NOTE_C4S 277
#define NOTE_D4F 277
#define NOTE_D4  294
#define NOTE_D4S 311
#define NOTE_E4F 311
#define NOTE_E4  330
#define NOTE_F4F 330
#define NOTE_E4S 349
#define NOTE_F4  349
#define NOTE_F4S 370
#define NOTE_G4F 370
#define NOTE_G4  392
#define NOTE_G4S 415
#define NOTE_A4F 415
#define NOTE_A4  440
#define NOTE_A4S 466
#define NOTE_B4F 466
#define NOTE_B4  494
#define NOTE_C5F 494
#define NOTE_B4S 523
#define NOTE_C5  523
#define NOTE_C5S 554
#define NOTE_D5F 554
#define NOTE_D5  587
#define NOTE_D5S 622
#define NOTE_E5F 622
#define NOTE_E5  659
#define NOTE_F5F 659
#define NOTE_E5S 698
#define NOTE_F5  698
#define NOTE_F5S 740
#define NOTE_G5F 740
#define NOTE_G5  784
#define NOTE_G5S 831
#define NOTE_A5F 831
#define NOTE_A5  880
#define NOTE_A5S 932
#define NOTE_B5F 832
#define NOTE_B5  988
#define NOTE_C6F 988
#define NOTE_B5S 1047
#define NOTE_C6  1047
#define NOTE_C6S 1109
#define NOTE_D6F 1109
#define NOTE_D6  1175
#define NOTE_D6S 1245
#define NOTE_E6F 1245
#define NOTE_E6  1319
#define NOTE_F6F 1319
#define NOTE_E6S 1397
#define NOTE_F6  1397
#define NOTE_F6S 1480
#define NOTE_G6F 1480
#define NOTE_G6  1568
#define NOTE_G6S 1661
#define NOTE_A6F 1661
#define NOTE_A6  1760
#define NOTE_A6S 1865
#define NOTE_B6F 1865
#define NOTE_B6  1976
#define NOTE_C7F 1976
#define NOTE_B6S 2093
#define NOTE_C7  2093
#define NOTE_C7S 2217
#define NOTE_D7F 2217
#define NOTE_D7  2349
#define NOTE_D7S 2489
#define NOTE_E7F 2489
#define NOTE_E7  2637
#define NOTE_F7F 2637
#define NOTE_E7S 2794
#define NOTE_F7  2794
#define NOTE_F7S 2960
#define NOTE_G7F 2960
#define NOTE_G7  3136
#define NOTE_G7S 3322
#define NOTE_A7F 3322
#define NOTE_A7  3520
#define NOTE_A7S 3729
#define NOTE_B7F 3729
#define NOTE_B7  3951
#define NOTE_C8F 3951
#define NOTE_B7S 4186
#define NOTE_C8  4186


/*************************************************
* Definitions
*************************************************/

class Tone
{
  public:
    void begin(uint8_t tonePin);
    bool isPlaying();
    void play(uint16_t frequency, uint32_t duration = 0);
    void stop();

  private:
    static uint8_t _tone_pin_count;
    uint8_t _pin;
    int8_t _timer;
};

#endif
